@@ -212,7 +212,7 @@ class ExternalResources(Container):
 
     @docval({'name': 'obj', 'type': (int, Object), 'doc': 'the Object to that uses the Key'},
             {'name': 'key', 'type': (int, Key), 'doc': 'the Key that the Object uses'})
-    def _add_external_reference(self, **kwargs):
+    def _add_object_key(self, **kwargs):
         """
         Specify that an object (i.e. container and field) uses a key to reference
         an external resource
@@ -332,6 +332,7 @@ class ExternalResources(Container):
 
         if not isinstance(key, Key):
             key = self._add_key(key)
+            self._add_object_key(object_field, key)
 
         if kwargs['resources_idx'] is not None and kwargs['resource_name'] is None and kwargs['resource_uri'] is None:
             resource_table_idx = kwargs['resources_idx']
@@ -358,10 +359,36 @@ class ExternalResources(Container):
 
         if add_entity:
             entity = self._add_entity(key, resource_table_idx, entity_id, entity_uri)
-            self._add_external_reference(object_field, key)
 
         return key, resource_table_idx, entity
 
+    @docval({'name': 'container', 'type': (str, AbstractContainer),
+             'doc': 'the Container/data object that is linked to resources/entities',
+             'default': None},
+            {'name': 'field', 'type': str,
+             'doc': 'the field of the Container',
+             'default': None})
+    def get_object_resources(self, **kwargs):
+        """
+        Get all entities/resources associated with an object
+        """
+        container = kwargs['container']
+        field = kwargs['field']
+
+        keys = []
+        entities = []
+        if container is not None and field is not None:
+            object_field = self._check_object_field(container, field)
+            # Find all keys associated with the object
+            for row_idx in self.object_keys.which(objects_idx=object_field.idx):
+                keys.append(self.object_keys['keys_idx', row_idx])
+            # Find all the entities/resources for each key.
+            for key_idx in keys:
+                entity_idx = self.entities.which(keys_idx=key_idx)
+                entities.append(self.entities.__getitem__(entity_idx[0]))
+            df = pd.DataFrame(entities, columns=['keys_idx', 'resource_idx', 'entity_id', 'entity_uri'])
+        return df
+
     @docval({'name': 'keys', 'type': (list, Key), 'default': None,
              'doc': 'the Key(s) to get external resource data for'},
             rtype=pd.DataFrame, returns='a DataFrame with keys and external resource data')
