@@ -27,6 +27,11 @@ def append_data(data, arg):
 
 
 def extend_data(data, arg):
+    """Add all the elements of the iterable arg to the end of data.
+
+    :param data: The array to extend
+    :type data: list, DataIO, np.ndarray, h5py.Dataset
+    """
     if isinstance(data, (list, DataIO)):
         data.extend(arg)
         return data
